package com.glxp.api.admin.controller.basic;

import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.entity.basic.BussinessChangeTypeEntity;
import com.glxp.api.admin.entity.basic.BussinessLocalTypeEntity;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.receipt.BussinessTypeFilterRequest;
import com.glxp.api.admin.service.basic.BussinessChangeTypeService;
import com.glxp.api.admin.service.basic.BussinessLocalTypeService;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class BussinessChangeTypeController {


    @Resource
    private BussinessChangeTypeService bussinessChangeTypeService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private BussinessLocalTypeService bussinessLocalTypeService;

    @GetMapping("/spms/bussinessChangeType/filter")
    public BaseResponse filterBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                            BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessChangeTypeEntity> bussinessTypeEntities;
        BussinessLocalTypeEntity bussinessLocalTypeEntity = bussinessLocalTypeService.findBTByAction(bussinessTypeFilterRequest.getAction());
        if (bussinessLocalTypeEntity == null) {
            return ResultVOUtils.error(500, "业务单据类型错误");
        }
        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(bussinessLocalTypeEntity.getAction());
        bussinessTypeEntities = bussinessChangeTypeService.findByAction(bussinessTypeFilterRequest.getAction());

        if (bussinessTypeEntities == null || bussinessTypeEntities.size() == 0) {
            //创建对应流转单据
            if (bussinessTypeFilterRequest.getType() != null
                    && "changeEnable".equals(bussinessTypeFilterRequest.getType())
            ) {
                if (ConstantType.TYPE_OUT.equals(bussinessTypeEntity.getMainAction())) {

                    bussinessTypeEntities = bussinessChangeTypeService.findByAction(bussinessTypeFilterRequest.getAction());
                    BussinessChangeTypeEntity bussinessChangeTypeEntity1 = new BussinessChangeTypeEntity();
                    bussinessChangeTypeEntity1.setIntro("出库自动补\"采购入库单\"");
                    bussinessChangeTypeEntity1.setIndex(1);
                    bussinessChangeTypeEntity1.setAction(bussinessTypeFilterRequest.getAction());
                    bussinessChangeTypeService.insertBusChangeTypee(bussinessChangeTypeEntity1);
                }
            }
        }
        return ResultVOUtils.success(bussinessTypeEntities);
    }


    @PostMapping("/spms/bussinessChangeType/delete")
    public BaseResponse deleteBussinessType(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        bussinessChangeTypeService.deleteById(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/spms/bussinessChangeType/update")
    public BaseResponse updateBussinessType(@RequestBody BussinessChangeTypeEntity bussinessTypeEntity,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessChangeTypeService.updateBusChangeType(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("更新成功");
    }


    @PostMapping("/spms/bussinessChangeType/insert")
    public BaseResponse insertBussinessType(@RequestBody BussinessChangeTypeEntity
                                                    bussinessTypeEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessChangeTypeService.insertBusChangeTypee(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("添加成功");
    }

}
