package com.glxp.api.admin.controller.basic;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.annotation.AuthRuleAnnotation;
import com.glxp.api.admin.constant.BussinessTypeUtils;
import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.entity.basic.BusDataEntity;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.req.inout.PostOrderRequest;
import com.glxp.api.admin.req.receipt.BussinessTypeFilterRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.basic.BasicThirdSysResponse;
import com.glxp.api.admin.res.basic.BussinessTypExportResponse;
import com.glxp.api.admin.res.basic.BussinessTypResponse;
import com.glxp.api.admin.res.basic.SrBtResponse;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.FileUtils;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 业务类型维护
 */

@RestController
public class BussinessTypeController {


    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private OrderService orderService;

    @GetMapping("/udiwms/bussinessType/filter")
    public BaseResponse filterBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessTypeEntity> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterList(bussinessTypeFilterRequest);
        PageInfo<BussinessTypeEntity> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessTypeEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/bussinessType/joinFilter")
    public BaseResponse filterJoinBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                                BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessTypResponse> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterJoinList(bussinessTypeFilterRequest);
        PageInfo<BussinessTypResponse> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessTypResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    //    @AuthRuleAnnotation("")
    @GetMapping("/udiwms/bussinessType/download")
    public BaseResponse downloadJoinBussinessType(BussinessTypeFilterRequest bussinessTypeFilterRequest,
                                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        bussinessTypeFilterRequest.setEnabled(true);
        List<BussinessTypResponse> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterJoinList(bussinessTypeFilterRequest);

        List<SrBtResponse> basicThirdSysResponses = bussinessTypeEntities.stream().map(item ->
        {
            SrBtResponse basicThirdSysResponse = new SrBtResponse();
            BeanUtils.copyProperties(item, basicThirdSysResponse);
            return basicThirdSysResponse;
        }).collect(Collectors.toList());

        PageInfo<BussinessTypResponse> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<SrBtResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicThirdSysResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/bussinessType/delete")
    public BaseResponse deleteBussinessType(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.selectById(id);
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setAction(bussinessTypeEntity.getAction());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        if (orderEntity != null) {
            return ResultVOUtils.error(500, "已存在该类型扫码单据，无法删除！");
        }


        if (bussinessTypeEntity.getLocalAction() != null && !"".equals(bussinessTypeEntity.getLocalAction())) {
            return ResultVOUtils.error(500, "该扫码单据类型已绑定业务单据类型，无法删除！");
        }


        bussinessTypeService.deleteById(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/bussinessType/update")
    public BaseResponse updateBussinessType(@RequestBody BussinessTypeEntity bussinessTypeEntity,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessTypeService.updateBussinessType(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("更新成功");
    }


    @PostMapping("/udiwms/bussinessType/insert")
    public BaseResponse insertBussinessType(@RequestBody BussinessTypeEntity bussinessTypeEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessTypeEntity.setAction("SC" + CustomUtil.getId());
            bussinessTypeService.insertBussinessType(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("添加成功");
    }


    //扫码单据类型文件导入
    @PostMapping("/udiwms/busstiness/file/upload")
    public BaseResponse uploadProducts(@RequestParam("file") List<MultipartFile> files) {
        for (int i = 0; i < files.size(); i++) {
            MultipartFile file = files.get(i);
            if (file.isEmpty()) {
                return ResultVOUtils.error(500, "上传第" + (i++) + "个文件失败");
            }
            try {
                InputStream inputStream = file.getInputStream();
                String json = FileUtils.readStream(inputStream);
                String[] list = json.split("\r\n");
                if (list != null && list.length > 0) {
                    for (String item : list) {
                        item.replace(" ", "");
                        String[] data = item.split(",");
                        BussinessTypeEntity bussinessTypeEntity = new BussinessTypeEntity();
                        bussinessTypeEntity.setIndex(Integer.parseInt(data[0]));
                        bussinessTypeEntity.setMainAction(data[1]);
                        bussinessTypeEntity.setAction(data[2]);
                        bussinessTypeEntity.setName(data[3]);
                        bussinessTypeService.insertIgnoreBussinessType(bussinessTypeEntity);
                    }
                }
                System.out.println(json.toUpperCase(Locale.ROOT));
            } catch (Exception e) {
                e.printStackTrace();
                return ResultVOUtils.error(500, "文件格式错误！");
            }
        }

        return ResultVOUtils.success("导入成功！");

    }


    //扫码单据类型导出
    @PostMapping("udiwms/busstiness/file/export")
    public void exportXml(HttpServletResponse res) {

        BussinessTypeFilterRequest bussinessTypeFilterRequest = new BussinessTypeFilterRequest();
        bussinessTypeFilterRequest.setEnabled(true);
        List<BussinessTypeEntity> bussinessTypeEntities;
        List<BussinessTypExportResponse> bussinessTypExportResponses = new ArrayList<>();
        bussinessTypeEntities = bussinessTypeService.filterList(bussinessTypeFilterRequest);
        for (BussinessTypeEntity bussinessTypeEntity : bussinessTypeEntities) {
            BussinessTypExportResponse bussinessTypExportResponse = new BussinessTypExportResponse();
            BeanUtils.copyProperties(bussinessTypeEntity, bussinessTypExportResponse);
            bussinessTypExportResponses.add(bussinessTypExportResponse);
        }
        String json = JSONObject.toJSON(bussinessTypExportResponses).toString();
        String fileName = "扫码单据类型导出" + CustomUtil.getId() + ".json";
        res.setHeader("Content-disposition", "attachment;fileName=" + fileName);
        res.setContentType("text/plain;charset=UTF-8");
        try {
            res.getOutputStream().write(json.getBytes(StandardCharsets.UTF_8));
        } catch (IOException e) {
            e.printStackTrace();
        }

//        return ResultVOUtils.success("文件生成成功！！");
    }


}
