package com.glxp.api.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.IOOrderStatusEntity;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.req.inout.OrderStatusFilterRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inout.DlOrderResponse;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.service.inout.IOOrderStatusService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
public class IOOrderStatusController {

    @Resource
    private OrderService orderService;
    @Resource
    private CodesService codesService;
    @Resource
    private OrderDetailService orderDetailService;
    @Resource
    private IOOrderStatusService ioOrderStatusService;

    @GetMapping("spms/inout/order/status")
    public BaseResponse listOrderStatus(OrderStatusFilterRequest orderStatusFilterRequest) {
        List<IOOrderStatusEntity> orderStatusEntities = ioOrderStatusService.filterOrderStatus(orderStatusFilterRequest);
        PageInfo<IOOrderStatusEntity> pageInfo;
        pageInfo = new PageInfo<>(orderStatusEntities);
        PageSimpleResponse<IOOrderStatusEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderStatusEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("spms/inout/order/dlByStatus")
    public BaseResponse dlByStatus(OrderStatusFilterRequest orderStatusFilterRequest) {
        List<DlOrderResponse> dlOrderResponses = new ArrayList<>();
        List<IOOrderStatusEntity> orderStatusEntities = ioOrderStatusService.filterOrderStatus(orderStatusFilterRequest);
        if (orderStatusEntities != null && orderStatusEntities.size() > 0) {
            for (IOOrderStatusEntity orderStatusEntity : orderStatusEntities) {
                DlOrderResponse dlOrderResponse = new DlOrderResponse();
                OrderEntity orderEntity = orderService.findById(orderStatusEntity.getOrderId());
                dlOrderResponse.setOrderEntity(orderEntity);
                List<ErpOrderEntity> orderDetailEntityList = orderDetailService.filterAllByOrderIdFk(orderStatusEntity.getOrderId());
                dlOrderResponse.setOrderDetails(orderDetailEntityList);
                List<WarehouseEntity> codes = codesService.findByReceiptId(orderStatusEntity.getOrderId());
                dlOrderResponse.setCodes(codes);
                dlOrderResponses.add(dlOrderResponse);
            }
        }
        return ResultVOUtils.success(dlOrderResponses);
    }

    //更新订单
    @PostMapping("/spms/inout/order/deleteStatus")
    public BaseResponse updateStatus(@RequestBody OrderStatusFilterRequest orderStatusFilterRequest,
                                           BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = ioOrderStatusService.deleteByOrderId(orderStatusFilterRequest.getOrderId());
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }
}
