package com.glxp.api.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.api.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.api.admin.entity.basic.BussinessOriginTypeEntity;
import com.glxp.api.admin.entity.basic.BussinessTypeEntity;
import com.glxp.api.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.api.admin.entity.business.StockOrderDetailEntity;
import com.glxp.api.admin.entity.business.StockOrderEntity;
import com.glxp.api.admin.entity.inout.*;
import com.glxp.api.admin.entity.thrsys.ThrOrderDetailEntity;
import com.glxp.api.admin.entity.thrsys.ThrOrderEntity;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.req.business.StockOrderDetailFilterRequest;
import com.glxp.api.admin.req.business.StockOrderFilterRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.api.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.api.admin.res.basic.BussinessOriginTypeResponse;
import com.glxp.api.admin.res.inout.ErpOrderResponse;
import com.glxp.api.admin.req.inout.*;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.service.basic.BussinessOriginTypeService;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.business.StockOrderDetailService;
import com.glxp.api.admin.service.business.StockOrderService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.*;
import com.glxp.api.admin.service.thrsys.ThrOrderDetailService;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import com.glxp.api.admin.thread.IoTransInoutService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.DateUtil;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class OrderDetailController {

    @Resource
    OrderDetailService myErpOrderService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    CodesService warehouseService;
    @Resource
    OrderService orderService;
    @Resource
    private UdiRelevanceService udiRelevanceService;

    @Resource
    private IoTransInoutService transInoutService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    private ThrOrderDetailService thrOrderDetailService;

    @Resource
    private StockOrderService stockOrderService;
    @Resource
    private StockOrderDetailService stockOrderDetailService;
    @Resource
    ContrastErpUtil contrastErpUtil;
    @Resource
    BussinessOriginTypeService bussinessOriginTypeService;

    //查询本地单据和第三方单据
    @GetMapping("/udiwms/erpOrder/filter")
    public BaseResponse filterErpOrder(FilterOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(filterErpOrderRequest.getBillAction(), "orderQueryUrl");
        if (basicThirdSysDetailEntity == null || !basicThirdSysDetailEntity.getEnabled()) {
            return ResultVOUtils.error(500, "第三方业务单据服务未启用");
        }

        if (basicThirdSysDetailEntity.getThirdSysFk() == null) {
            return ResultVOUtils.error(500, "第三方业务单据服务未关联");
        }

        if (filterErpOrderRequest.getIsDownThrSys() != null && filterErpOrderRequest.getIsDownThrSys()) {
            return new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);
        }

//        if (filterErpOrderRequest.getIsLocal() != null && filterErpOrderRequest.getIsLocal()) {
//            return getLoaclData(filterErpOrderRequest);
//        }

        if (basicThirdSysDetailEntity.getLocalAction() != null && basicThirdSysDetailEntity.getThirdSysFk() == null) {
            filterErpOrderRequest.setLocalAction(basicThirdSysDetailEntity.getLocalAction());
            return getLoaclData(filterErpOrderRequest);
        }

        if (basicThirdSysDetailEntity.getFromType() != null && basicThirdSysDetailEntity.getFromType() == 0) {
            if (basicThirdSysDetailEntity.getValue() == null) {
                return ResultVOUtils.error(500, "业务单据查询接口未定义");
            }
            filterErpOrderRequest.setBillAction(basicThirdSysDetailEntity.getThirdAction());
            BaseResponse<PageSimpleResponse<ErpOrderResponse>> baseResponse = new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);

//
            if (basicThirdSysDetailEntity.getLocalAction() != null) {
                if (baseResponse.getData().getList() != null) {
                    BaseResponse<PageSimpleResponse<ErpOrderResponse>> loca = getLoaclData(filterErpOrderRequest);
                    baseResponse.getData().getList().addAll(loca.getData().getList());
                }
            }


            return baseResponse;

        } else {
            FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
            BeanUtils.copyProperties(filterErpOrderRequest, filterThrOrderRequest);
            filterThrOrderRequest.setBillAction(basicThirdSysDetailEntity.getThirdAction());
            filterThrOrderRequest.setPage(filterErpOrderRequest.getPage());
            filterThrOrderRequest.setLimit(filterErpOrderRequest.getLimit());
            List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
            List<ThrOrderEntity> data = thrOrderService.filterThrOrder(filterThrOrderRequest);
            if (data != null && data.size() > 0) {
                for (ThrOrderEntity thrOrderEntity : data) {
                    FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                    filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                    List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                    ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                    BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                    List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                    if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                        for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                            ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                            BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                            subErpOrders.add(subErpOrder);
                        }
                    }
                    erpOrderResponse.setSubErpOrders(subErpOrders);
                    erpOrderResponses.add(erpOrderResponse);
                }
            }
            if (basicThirdSysDetailEntity.getLocalAction() != null) {
                BaseResponse<PageSimpleResponse<ErpOrderResponse>> loca = getLoaclData(filterErpOrderRequest);
                erpOrderResponses.addAll(loca.getData().getList());
            }
            PageInfo<ThrOrderEntity> pageInfo;
            pageInfo = new PageInfo<>(data);
            PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(erpOrderResponses);
            return ResultVOUtils.success(pageSimpleResponse);

        }
    }

    @GetMapping("/udiwms/erpOrder/addStockOrder")
    public BaseResponse addStockOrder(FilterOrderRequest filterErpOrderRequest,
                                      BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ThrOrderEntity thrOrderEntity = thrOrderService.findById(String.valueOf(filterErpOrderRequest.getId()));
        if (thrOrderEntity == null) {
            return ResultVOUtils.error(ResultEnum.DATA_NOT);
        }

        StockOrderEntity temp = stockOrderService.findByBillNo(thrOrderEntity.getBillNo() + "");
        if (temp != null) {
            return ResultVOUtils.error(500, "单据已被选入，请勿重复选入！");
        }


        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        BeanUtils.copyProperties(thrOrderEntity, stockOrderEntity);
        String billNo = CustomUtil.getId();
        stockOrderEntity.setBillNo(billNo);
        BussinessOriginTypeResponse bussinessOriginTypeResponse = bussinessOriginTypeService.finAllByAction(thrOrderEntity.getBillType());

        stockOrderEntity.setBillType(bussinessOriginTypeResponse.getLocalAction());
        stockOrderEntity.setId(null);
        stockOrderEntity.setStatus(ConstantStatus.SORDER_ALLOC);//未提交
        stockOrderEntity.setThirdOrderFk(thrOrderEntity.getBillNo());
        stockOrderEntity.setSourceType(ConstantStatus.ST_THIRD_SEL);//外部提交
        stockOrderEntity.setPrintStatus(ConstantStatus.SORDER_PRINT_UN);//未打印
        stockOrderEntity.setBilldate(DateUtil.formatDate(new Date()));

        stockOrderEntity = stockOrderService.findByBillNo(billNo);
        FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
        filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
        List<ThrOrderDetailEntity> thrOrderDetailEntities =
                thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
        if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
            List<StockOrderDetailEntity> stockOrderDetailEntityList = new ArrayList<>();
            for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
                UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectByThirdId(thrOrderDetailEntity.getProductId(), thrOrderDetailEntity.getThirdSysFk());

                if (udiRelevanceEntity == null) {
                    return ResultVOUtils.error(500, "单据里含有未维护的产品信息，选入失败！");
                }
                BeanUtils.copyProperties(thrOrderDetailEntity, stockOrderDetailEntity);
                stockOrderDetailEntity.setProductId(udiRelevanceEntity.getId());
                stockOrderDetailEntity.setCount(thrOrderDetailEntity.getCount() + "");
                stockOrderDetailEntity.setReCount(thrOrderDetailEntity.getReCount() + "");
                stockOrderDetailEntity.setOrderIdFk(stockOrderEntity.getId());
                stockOrderDetailEntity.setSweepCount(0 + "");
                stockOrderDetailEntityList.add(stockOrderDetailEntity);
            }
            stockOrderDetailService.insertStockOrderDetails(stockOrderDetailEntityList);
        }
        boolean b = stockOrderService.insertStockOrder(stockOrderEntity);
        return ResultVOUtils.success();
    }

    //只查询第三方单据
    @GetMapping("/udiwms/thirdOrder/filter")
    public BaseResponse filterThirdOrder(FilterOrderRequest filterErpOrderRequest,
                                         BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessOriginTypeService.findSysByAction(filterErpOrderRequest.getBillAction(), "orderQueryUrl");
        if (basicThirdSysDetailEntity == null || !basicThirdSysDetailEntity.getEnabled()) {
            return ResultVOUtils.error(500, "第三方业务单据服务未启用");
        }

        if (basicThirdSysDetailEntity.getThirdSysFk() == null) {
            return ResultVOUtils.error(500, "第三方业务单据服务未关联");
        }

        if (filterErpOrderRequest.getIsDownThrSys() != null && filterErpOrderRequest.getIsDownThrSys()) {
            return new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);
        }
        if (basicThirdSysDetailEntity.getFromType() != null && basicThirdSysDetailEntity.getFromType() == 0) {
            if (basicThirdSysDetailEntity.getValue() == null) {
                return ResultVOUtils.error(500, "业务单据查询接口未定义");
            }
            BaseResponse<PageSimpleResponse<ErpOrderResponse>> baseResponse = new ErpOrderClient(basicThirdSysDetailEntity.getValue()).getErpOrderResponse(filterErpOrderRequest);
            return baseResponse;
        } else {
            FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
            BeanUtils.copyProperties(filterErpOrderRequest, filterThrOrderRequest);
            filterThrOrderRequest.setPage(filterErpOrderRequest.getPage());
            filterThrOrderRequest.setLimit(filterErpOrderRequest.getLimit());
            List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
            List<ThrOrderEntity> data = thrOrderService.filterThrOrder(filterThrOrderRequest);
            if (data != null && data.size() > 0) {
                for (ThrOrderEntity thrOrderEntity : data) {
                    FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                    filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                    List<ThrOrderDetailEntity> thrOrderDetailEntities = thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                    ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                    BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                    List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                    if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                        for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                            ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                            BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                            subErpOrders.add(subErpOrder);
                        }
                    }
                    erpOrderResponse.setSubErpOrders(subErpOrders);
                    erpOrderResponses.add(erpOrderResponse);
                }
            }
            if (basicThirdSysDetailEntity.getLocalAction() != null) {
                BaseResponse<PageSimpleResponse<ErpOrderResponse>> loca = getLoaclData(filterErpOrderRequest);
                erpOrderResponses.addAll(loca.getData().getList());
            }
            PageInfo<ThrOrderEntity> pageInfo;
            pageInfo = new PageInfo<>(data);
            PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(erpOrderResponses);
            return ResultVOUtils.success(pageSimpleResponse);

        }
    }


    public BaseResponse getLoaclData(FilterOrderRequest filterErpOrderRequest) {
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        BeanUtils.copyProperties(filterErpOrderRequest, stockOrderFilterRequest);
        stockOrderFilterRequest.setBillType(filterErpOrderRequest.getLocalAction());
        stockOrderFilterRequest.setPage(filterErpOrderRequest.getPage());
        stockOrderFilterRequest.setLimit(filterErpOrderRequest.getLimit());
        List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
        List<StockOrderEntity> data = stockOrderService.filterListOr(stockOrderFilterRequest);
        if (data != null && data.size() > 0) {
            for (StockOrderEntity thrOrderEntity : data) {
                StockOrderDetailFilterRequest filterThrOrderDetailRequest = new StockOrderDetailFilterRequest();
                filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                List<StockOrderDetailEntity> thrOrderDetailEntities = stockOrderDetailService.filterStockOrderDetail(filterThrOrderDetailRequest);
                ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                    for (StockOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                        ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                        BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                        subErpOrder.setRelId(thrOrderDetailEntity.getProductId());
                        subErpOrders.add(subErpOrder);
                    }
                }
                erpOrderResponse.setSubErpOrders(subErpOrders);
                erpOrderResponses.add(erpOrderResponse);
            }
        }
        PageInfo<StockOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(data);
        PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpOrderResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/warehouse/getMyErpOrders")
    public BaseResponse getMyErpOrders(FilterErpOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getErpOrderId() == null || filterErpOrderRequest.getErpOrderId().equals("")) {
            filterErpOrderRequest.setErpOrderId("null");
        }
        List<ErpOrderEntity> erpInfoEntityList = myErpOrderService.filterMyErpDetailOrder(filterErpOrderRequest);
        PageInfo<ErpOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(erpInfoEntityList);
        PageSimpleResponse<ErpOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/deleteErpOrder")
    public BaseResponse deleteErpOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = myErpOrderService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }

    @GetMapping(value = "/warehouse/inout/order/detail")
    public BaseResponse orderDetail(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            filterErpOrderRequest.setOrderId("1");
        }
        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterMyErpOrder(filterErpOrderRequest);
        return ResultVOUtils.success(erpOrderEntities);
    }

    @GetMapping(value = "/warehouse/inout/order/uploadOrder")
    public BaseResponse uploadOrder(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            return ResultVOUtils.error(500, "上传失败");
        }

        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterMyErpOrder(filterErpOrderRequest);
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        if (erpOrderEntities != null) {
            for (int i = 0; i < erpOrderEntities.size(); i++) {
                if (erpOrderEntities.get(i).getErpOrderId() == null) {
                    return ResultVOUtils.error(500, "批次号不匹配，请检查后修改！");
                }

                if (erpOrderEntities.get(i).getErpOrderId().contains(",")) {
                    return ResultVOUtils.error(500, "无法修改合并单据！");
                }

                if (erpOrderEntities.get(i).getCount() > erpOrderEntities.get(i).getErpCount()) {
                    return ResultVOUtils.error(500, "扫码数量大于实际数量,无法修改！");
                }
            }
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(orderEntity.getAction(), "orderModifyUrl");
        if (basicThirdSysDetailEntity == null) {
            return ResultVOUtils.error(500, "ERP出入库单据查询接口地址未定义");
        }
        String url = basicThirdSysDetailEntity.getValue();
        Map<String, List<ErpOrderEntity>> listMap = erpOrderEntities.stream().collect(Collectors.groupingBy(ErpOrderEntity::getErpOrderId));
        BaseResponse baseResponse = new ErpOrderClient().postBill(listMap, orderEntity.getAction(), url);
        if (baseResponse != null && baseResponse.getCode() == 20000) {
            orderService.updateExportStatus(orderEntity.getId(), ConstantStatus.ORDER_EXPORT_ED);
            return ResultVOUtils.success("修改成功");
        } else {
            if (baseResponse != null) {
                return ResultVOUtils.error(500, baseResponse.getMessage());
            }
            return ResultVOUtils.error(500, "修改失败");
        }
    }


    @GetMapping("/warehouse/dlErpStock1")
    public BaseResponse dlErpStock() {
//        dlErpStockService.dlErpStock();
        return ResultVOUtils.success("后台已自动更新，请稍后刷新查看！");
    }

    @PostMapping("/warehouse/selectErpOrder")
    public BaseResponse selectErpOrder(@RequestBody ImportErpOrderRequest importErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ErpOrderResponse> docids = importErpOrderRequest.getPurchaseinResponses();
        String docidStr = "";
        if (docids != null && docids.size() > 0) {
            for (ErpOrderResponse temp : docids) {
                docidStr = docidStr + "," + temp.getBillNo();
            }
            docidStr = docidStr.substring(1);
        }
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (docids != null && docids.size() > 0) {
            for (ErpOrderResponse purchaseinResponse : docids) {
                for (ErpOrderResponse.SubErpOrder subPurchase : purchaseinResponse.getSubErpOrders()) {
                    ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                    erpOrderEntity.setErpOrderId(purchaseinResponse.getBillNo());
                    erpOrderEntity.setCompanyid(purchaseinResponse.getCorpId());
                    erpOrderEntity.setCompanyname(purchaseinResponse.getCorpName());
                    erpOrderEntity.setCredate(purchaseinResponse.getBilldate());
                    erpOrderEntity.setGoodsid(subPurchase.getProductId());
                    erpOrderEntity.setGoodsname(subPurchase.getProductName());
                    erpOrderEntity.setRelIdFk(subPurchase.getRelId());
                    erpOrderEntity.setBatchNo(subPurchase.getBatchNo());
                    if (subPurchase.getCount() != null)
                        erpOrderEntity.setErpCount(Math.abs(Integer.parseInt(subPurchase.getCount())));
                    if (subPurchase.getReCount() != null)
                        erpOrderEntity.setReCount(Math.abs(Integer.parseInt(subPurchase.getReCount())));
                    erpOrderEntity.setGoodsunit(subPurchase.getSpec());
                    erpOrderEntities.add(erpOrderEntity);
                }
            }
        }
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            ErpOrderEntity temp = erpOrderEntities.get(0);
            for (int i = 0; i < erpOrderEntities.size(); i++) {
                if (!temp.getCompanyname().equals(erpOrderEntities.get(i).getCompanyname())) {
                    return ResultVOUtils.error(500, "选择Erp订单往来单位不一致！");
                }
            }

            OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
            orderFilterRequest.setId(importErpOrderRequest.getOrderId());
            OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());
            ErpOrderEntity exitErp = new ContrastErpUtil().vailErpExit(myErpOrderService, erpOrderEntities, importErpOrderRequest.getOrderId());
            if (exitErp != null) {
                String errMsg = "所选ERP单" + exitErp.getErpOrderId() + "已被其他出入库单" + exitErp.getOrderIdFk() + "绑定";
                orderEntity.setRemark(errMsg);
                orderService.updateOrder(orderEntity);
                return ResultVOUtils.error(500, errMsg);
            } else {
                orderEntity.setErpFk(docidStr);
                orderService.updateOrder(orderEntity);
            }
            List<ErpOrderEntity> returnOrders = contrastErpUtil.transErp(erpOrderEntities, udiRelevanceService, importErpOrderRequest.getOrderId(), bussinessTypeEntity);
            List<WarehouseEntity> warehouseEntityList = warehouseService.findByReceiptId(importErpOrderRequest.getOrderId());
            contrastErpUtil.contrastErp(udiInfoService, udiRelevanceService,
                    myErpOrderService, orderService, warehouseEntityList, systemParamConfigService, returnOrders);
        } else {
            return ResultVOUtils.error(500, "ERP订单不能为空");
        }

        return ResultVOUtils.success("导入成功");
    }

    @PostMapping("/udiwms/erpOrder/updateBind")
    public BaseResponse updateBind(@RequestBody ErpOrderEntity erpOrderEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        ErpOrderEntity erpOrderEntity1 = myErpOrderService.findById(erpOrderEntity.getId() + "");
        warehouseService.updateRelId(erpOrderEntity.getBindRlFk(), erpOrderEntity1.getNameCode(), erpOrderEntity1.getOrderIdFk());
        boolean b = myErpOrderService.updateErpOrder(erpOrderEntity);
        if (b) {
            return ResultVOUtils.success("绑定成功");
        } else {
            return ResultVOUtils.error(500, "绑定失败");
        }
    }


    @PostMapping("/udiwms/erpOrder/updateBindSup")
    public BaseResponse updateBindSup(@RequestBody ErpOrderEntity erpOrderEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        erpOrderEntity.setBindSupStatus(ConstantStatus.ORDER_DETAIL_SUP_ONE);
        boolean b = myErpOrderService.updateErpOrder(erpOrderEntity);
        erpOrderEntity = myErpOrderService.findById(erpOrderEntity.getId() + "");

        warehouseService.updateSupId(erpOrderEntity.getSupId(), erpOrderEntity.getNameCode(), erpOrderEntity.getOrderIdFk());
        if (b) {
            return ResultVOUtils.success("绑定成功");
        } else {
            return ResultVOUtils.error(500, "绑定失败");
        }
    }

    @PostMapping("/warehouse/repeatCheck")
    public BaseResponse repeatCheck(@RequestBody FilterErpOrderRequest filterErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        transInoutService.repeatCheck(orderEntity.getId());
        return ResultVOUtils.success("后台已开始校验，请稍后刷新重试！");
    }


}
