package com.glxp.api.admin.thread;

import com.glxp.api.admin.config.WebSocketServer;
import com.glxp.api.admin.constant.Constant;
import com.glxp.api.admin.constant.ConstantStatus;
import com.glxp.api.admin.constant.ConstantType;
import com.glxp.api.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.api.admin.controller.inout.utils.DataTransUtil;
import com.glxp.api.admin.controller.inout.utils.InoutUtils;
import com.glxp.api.admin.entity.basic.*;
import com.glxp.api.admin.entity.business.StockOrderDetailEntity;
import com.glxp.api.admin.entity.business.StockOrderEntity;
import com.glxp.api.admin.entity.info.CompanyEntity;
import com.glxp.api.admin.entity.inout.ErpOrderEntity;
import com.glxp.api.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.api.admin.entity.inout.OrderEntity;
import com.glxp.api.admin.entity.inout.WarehouseEntity;
import com.glxp.api.admin.entity.inventory.InvProductDetailEntity;
import com.glxp.api.admin.httpclient.ErpOrderClient;
import com.glxp.api.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.api.admin.req.business.StockOrderFilterRequest;
import com.glxp.api.admin.req.inout.CommitRequest;
import com.glxp.api.admin.req.inout.OrderFilterRequest;
import com.glxp.api.admin.req.inout.PostOrderRequest;
import com.glxp.api.admin.req.inout.WarehouseQueryRequest;
import com.glxp.api.admin.req.inventory.FilterInvProductDetailRequest;
import com.glxp.api.admin.res.basic.UdiRelevanceResponse;
import com.glxp.api.admin.service.basic.BasicUnitMaintainService;
import com.glxp.api.admin.service.basic.BussinessTypeService;
import com.glxp.api.admin.service.basic.UdiInfoService;
import com.glxp.api.admin.service.basic.UdiRelevanceService;
import com.glxp.api.admin.service.business.StockOrderDetailService;
import com.glxp.api.admin.service.business.StockOrderService;
import com.glxp.api.admin.service.info.SystemParamConfigService;
import com.glxp.api.admin.service.inout.CodesService;
import com.glxp.api.admin.service.inout.CodesTempService;
import com.glxp.api.admin.service.inout.OrderDetailService;
import com.glxp.api.admin.service.inout.OrderService;
import com.glxp.api.admin.service.inventory.InvProductDetailService;
import com.glxp.api.admin.service.thrsys.ThrDataService;
import com.glxp.api.admin.service.thrsys.ThrOrderService;
import com.glxp.api.admin.util.CustomUtil;
import com.glxp.api.admin.util.FilterUdiUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class IoTransInoutService {

    private static final Logger logger = LoggerFactory.getLogger(IoTransInoutService.class);
    @Resource
    private CodesService warehouseService;
    @Resource
    private OrderService orderService;
    @Resource
    private OrderDetailService myErpOrderService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    private UdiInfoService udiInfoService;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;

    @Resource
    private CodesTempService codesTempService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    ThrDataService thrDataService;
    @Resource
    InvProductsTrService invProductsTrService;
    @Resource
    InoutUtils inoutUtils;
    @Resource
    @Lazy
    ContrastErpUtil contrastErpUtil;
    @Resource
    StockOrderService stockOrderService;
    @Resource
    StockOrderDetailService stockOrderDetailService;
    @Resource
    BasicUnitMaintainService basicUnitMaintainService;
    @Resource
    InvProductDetailService invProductDetailService;

    @Async
    public void creatOrderAsycn(PostOrderRequest postOrderRequest, Integer fromType) {
        creatOrder(postOrderRequest, fromType);
    }

    //生成订单，插入零时表
    public void creatOrder(PostOrderRequest postOrderRequest, Integer fromType) {
        List<PostOrderRequest.PostOrder> postOrders = postOrderRequest.getPostOrder();
        if (postOrders != null && postOrders.size() > 0) {
            for (int i = 0; i < postOrders.size(); i++) {
                PostOrderRequest.PostOrder postOrder = postOrders.get(i);
                List<WarehouseEntity> warehouseEntityList = postOrder.getCodes();
                if (warehouseEntityList != null && warehouseEntityList.size() > 0) {

                    //解析条码
                    for (WarehouseEntity temp : warehouseEntityList) {
                        UdiEntity udiEntity = FilterUdiUtils.getUdi(temp.getCode());
                        temp.setNameCode(udiEntity.getUdi());
                    }


                    WarehouseEntity warehouseEntity = warehouseEntityList.get(0);
                    String orderId = System.currentTimeMillis() + i + "";
                    if (warehouseEntity.getOrderId() != null) {
                        orderId = warehouseEntity.getOrderId();
                    }

                    OrderEntity orderSaveRequest = new OrderEntity();
                    orderSaveRequest.setId(orderId);
                    if (warehouseEntity.getActDate() != null) {
                        orderSaveRequest.setActDate(warehouseEntity.getActDate());
                    }
                    orderSaveRequest.setAction(warehouseEntity.getAction());
                    orderSaveRequest.setActor(warehouseEntity.getActor());
                    orderSaveRequest.setCorpOrderId(warehouseEntity.getCorpOrderId());
                    orderSaveRequest.setFromCorpId(warehouseEntity.getFromCorpId());
                    orderSaveRequest.setMainAction(warehouseEntity.getMainAction());
                    orderSaveRequest.setFromCorp(warehouseEntity.getFromCorp());
                    if (fromType != null) {
                        orderSaveRequest.setFromType(fromType);
                    } else {
                        if (postOrder.getErpOrders() != null && postOrder.getErpOrders().size() > 0)
                            orderSaveRequest.setFromType(ConstantStatus.FROM_PDAED);
                        else
                            orderSaveRequest.setFromType(ConstantStatus.FROM_PDAUN);
                    }

                    orderSaveRequest.setStatus(ConstantStatus.ORDER_STATUS_PROCESS);
                    orderSaveRequest.setExportStatus(ConstantStatus.ORDER_EXPORT_UN);
                    orderSaveRequest.setStockCheckFk(postOrder.getStockIdFk());
                    orderSaveRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                    orderSaveRequest.setSignStatus(ConstantStatus.ORDER_SIGN_UN);
                    List<String> docidsList = postOrder.getErpOrders();
                    String docidStr = "";
                    if (docidsList != null) {
                        for (String key : docidsList) {
                            docidStr = docidStr + "," + key;
                        }
                        if (docidStr.length() > 1) {
                            docidStr = docidStr.substring(1);
                            orderSaveRequest.setErpFk(docidStr);
                        }
                    }
                    orderService.insertOrder(orderSaveRequest);
                    for (int index = 0; index < warehouseEntityList.size(); index++) {
                        warehouseEntityList.get(index).setOrderId(orderId);
                    }
                    codesTempService.insertCodesTemp(warehouseEntityList);
                }
            }
        }
    }

    //web端新增订单，
    @Async
    public void creatOrderWeb(PostOrderRequest postOrderRequest) {
        List<PostOrderRequest.PostOrder> postOrders = postOrderRequest.getPostOrder();
        if (postOrders != null && postOrders.size() > 0) {
            for (int i = 0; i < postOrders.size(); i++) {
                PostOrderRequest.PostOrder postOrder = postOrders.get(i);
                List<WarehouseEntity> warehouseEntityList = postOrder.getCodes();
                if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
                    if (StringUtils.isNotBlank(postOrderRequest.getOldOrderId())) {
                        orderService.deleteById(postOrderRequest.getOldOrderId());
                        myErpOrderService.deleteByOrderId(postOrderRequest.getOldOrderId());
                    }
                    WarehouseEntity warehouseEntity = warehouseEntityList.get(0);
                    String orderId = System.currentTimeMillis() + i + "";
                    OrderEntity orderSaveRequest = new OrderEntity();
                    orderSaveRequest.setId(orderId);
                    if (warehouseEntity.getActDate() != null) {
                        orderSaveRequest.setActDate(warehouseEntity.getActDate());
                    }
                    orderSaveRequest.setAction(warehouseEntity.getAction());
                    orderSaveRequest.setActor(warehouseEntity.getActor());
                    orderSaveRequest.setCorpOrderId(warehouseEntity.getCorpOrderId());
                    orderSaveRequest.setFromCorpId(warehouseEntity.getFromCorpId());
                    orderSaveRequest.setMainAction(warehouseEntity.getMainAction());
                    orderSaveRequest.setFromCorp(warehouseEntity.getFromCorp());
                    orderSaveRequest.setFromType(ConstantStatus.FROM_WEBNEW);
                    orderSaveRequest.setStatus(postOrderRequest.getSubmitType() == 0 ?
                            ConstantStatus.ORDER_STATUS_PROCESS : ConstantStatus.ORDER_STATUS_TEMP_SAVE);
                    orderSaveRequest.setExportStatus(ConstantStatus.ORDER_EXPORT_UN);
                    orderSaveRequest.setStockCheckFk(postOrder.getStockIdFk());
                    orderSaveRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                    orderSaveRequest.setSignStatus(ConstantStatus.ORDER_SIGN_UN);
                    List<String> docidsList = postOrder.getErpOrders();
                    String docidStr = "";
                    if (docidsList != null) {
                        for (String key : docidsList) {
                            docidStr = docidStr + "," + key;
                        }
                        if (docidStr.length() > 1) {
                            docidStr = docidStr.substring(1);
                            orderSaveRequest.setErpFk(docidStr);
                        }
                    }
                    orderService.insertOrder(orderSaveRequest);
                    for (int index = 0; index < warehouseEntityList.size(); index++) {
                        warehouseEntityList.get(index).setOrderId(orderId);
                    }
                    codesTempService.insertCodesTemp(warehouseEntityList);
                }
            }
        }
    }

    //查找等待处理单据，转入正式表
    @Async
    public void transInout() {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setStatus(ConstantStatus.ORDER_STATUS_PROCESS);
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        for (int i = 0; i < orderEntities.size(); i++) {
            OrderEntity orderEntity = orderEntities.get(i);
            orderEntity = orderService.findById(orderEntity.getId());
            if (orderEntity.getStatus() == ConstantStatus.ORDER_STATUS_PROCESS) {
                transInout(orderEntity);
            }

        }
    }
    public void transInout(OrderEntity orderEntity) {

        if (orderEntity.getFromCorp() == null) {
            orderEntity.setRemark("未选择往来单位");
            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_FAIL);
            orderService.updateOrder(orderEntity);
            return;
        }

        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());
        if (bussinessTypeEntity == null || !bussinessTypeEntity.getEnable()) {
            orderEntity.setRemark("单据类型未找到或单据类型未启用");
            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_FAIL);
            orderService.updateOrder(orderEntity);
            return;
        }
        if (bussinessTypeEntity.isGenUnit()) {
            BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
            basicUnitMaintainEntity.setErpId(CustomUtil.getUnitId());
            basicUnitMaintainEntity.setCorpType(ConstantStatus.CORP_SICK);
            basicUnitMaintainEntity.setName(orderEntity.getFromCorp());
            orderEntity.setFromCorpId(basicUnitMaintainEntity.getErpId());
            orderService.updateOrder(orderEntity);
            basicUnitMaintainService.insertBasicUnitMaintain(basicUnitMaintainEntity);
        }


        //校验是否重码
        WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
        warehouseQueryRequest.setOrderId(orderEntity.getId());
        List<WarehouseEntity> filterList = codesTempService.findByOrderId(warehouseQueryRequest);
        List<WarehouseEntity> insertList = new ArrayList<>();
        for (int j = 0; j < filterList.size(); j++) {
            WarehouseEntity warehouseEntity = filterList.get(j);
            String result = inoutUtils.filterCode(warehouseService, warehouseEntity, systemParamConfigService);
            if (result == null) {
                //计算扫码数量
                insertList.add(warehouseEntity);
            } else {
                orderService.updateOrderStatus(warehouseEntity.getOrderId(), ConstantStatus.ORDER_STATUS_FAIL);
                orderService.updateRemark(orderEntity.getId(), result);
                insertList.clear();
                return;
            }

            UdiRelevanceResponse udiRelevanceResponse = udiRelevanceService.selectByNameCode(warehouseEntity.getNameCode());
            if (udiRelevanceResponse == null) {
                orderService.updateOrderStatus(warehouseEntity.getOrderId(), ConstantStatus.ORDER_STATUS_FAIL);
                orderService.updateRemark(orderEntity.getId(), "udi码：" + warehouseEntity.getCode() + "该产品信息未维护！");
                return;
            } else if (udiRelevanceResponse.getIsUseDy() == 0 && udiRelevanceResponse.getDiType() == ConstantStatus.DITYPE_SYDY) {
                orderService.updateOrderStatus(warehouseEntity.getOrderId(), ConstantStatus.ORDER_STATUS_FAIL);
                orderService.updateRemark(orderEntity.getId(), "udi码：" + warehouseEntity.getCode() + "该产品不允许以使用单元入库！");
                return;
            }
        }

        if (insertList.size() > 0) {
            warehouseService.insertWarehouses(insertList);
            //未所有条码关联基础信息关联表ID
            List<WarehouseEntity> warehouseEntityList = warehouseService.findByReceiptId(orderEntity.getId());
            for (WarehouseEntity warehouseEntity : warehouseEntityList) {
                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setNameCode(warehouseEntity.getNameCode());
                List<UdiRelevanceResponse> udiRelevanceEntities = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
                if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 1) {
                    warehouseEntity.setRelId(null);
                    warehouseEntity.setStatus(ConstantStatus.ORDER_DETAIL_DI_MUTI);
                } else if (udiRelevanceEntities == null || udiRelevanceEntities.size() == 0) {
                    warehouseEntity.setRelId(null);
                    warehouseEntity.setStatus(2);
                } else {
                    warehouseEntity.setStatus(0);
                    warehouseEntity.setRelId(udiRelevanceEntities.get(0).getId() + "");
                }
                warehouseService.updateCode(warehouseEntity);
            }


            if (orderEntity.getErpFk() != null && !"".equals(orderEntity.getErpFk())) {//已经绑定校验单据号，等待校验
                orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_CHECK);
            } else {
                //生成单据详情
                inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, warehouseEntityList);
                if (bussinessTypeEntity.getThirdSysFk() == null && bussinessTypeEntity.getLocalAction() == null) {//不需要校验
                    checkMutiDi(orderEntity.getId());   //校验一个DI是否绑定多个产品ID
                    orderEntity = orderService.findById(orderEntity.getId());
                    if (orderEntity.getContrastStatus() != ConstantStatus.ORDER_CHECK_FAIL) {
                        orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_SUCCESS);
                        invProductsTrService.genInvProducts(orderEntity.getId());
                    }
                } else {//需要校验，等待校验
                    orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_CHECK);
                }
            }

            //删除临时表
            WarehouseQueryRequest tempQuery = new WarehouseQueryRequest();
            tempQuery.setOrderId(orderEntity.getId());
            List<WarehouseEntity> temps = codesTempService.findByOrderId(tempQuery);
            List<Long> ids = new ArrayList<>();
            for (WarehouseEntity w : temps) {
                ids.add(w.getId());
            }
            codesTempService.deleteCodesTemp(ids);
            insertList.clear();
            transStatus(orderEntity.getId());
        }
    }

    //转入正式表后，更换单据状态，等待校验或已完成
    public void transStatus(String orderId) {
        //todo 改成等待校验，选择校验
        OrderEntity orderEntity = orderService.findById(orderId);
        List<WarehouseEntity> warehouseEntityList = warehouseService.findByReceiptId(orderId);
        myErpOrderService.deleteByOrderId(orderEntity.getId());
        //为所有条码关联基础信息关联表ID
        for (WarehouseEntity warehouseEntity : warehouseEntityList) {
            FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
            filterUdiInfoRequest.setNameCode(warehouseEntity.getNameCode());
            List<UdiRelevanceResponse> udiRelevanceEntities = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
            if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 1) {
                warehouseEntity.setRelId(null);
                warehouseEntity.setStatus(ConstantStatus.ORDER_DETAIL_DI_MUTI);
            } else if (udiRelevanceEntities == null || udiRelevanceEntities.size() == 0) {
                warehouseEntity.setRelId(null);
                warehouseEntity.setStatus(2);
            } else {
                warehouseEntity.setStatus(0);
                warehouseEntity.setRelId(udiRelevanceEntities.get(0).getId() + "");
                String supId = checkOriginExitSup(warehouseEntity);
                if (supId != null) {
                    warehouseEntity.setSupId(supId);
                } else {
                    supId = checkInvExitSup(warehouseEntity);
                    warehouseEntity.setSupId(supId);
                }
            }
            warehouseService.updateCode(warehouseEntity);
        }
        String action = orderEntity.getAction();
        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(action);
        if (!bussinessTypeEntity.isCheckEnable()) {
            orderEntity = orderService.findById(orderEntity.getId());
            unCheckFinish(orderEntity, warehouseEntityList);

        } else {

            //todo 判断单据来源是否需要校验
            if (orderEntity.getFromType() == ConstantStatus.FROM_UDIMS) {
                if (!bussinessTypeEntity.isCheckUdims()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            } else if (orderEntity.getFromType() == ConstantStatus.FROM_WEBNEW) {
                if (!bussinessTypeEntity.isCheckWebNew()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            } else if (orderEntity.getFromType() == ConstantStatus.FROM_PDAED) {
                if (!bussinessTypeEntity.isCheckPdaEd()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            } else if (orderEntity.getFromType() == ConstantStatus.FROM_PDAUN) {
                if (!bussinessTypeEntity.isCheckPdaUn()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            } else if (orderEntity.getFromType() == ConstantStatus.FROM_PC) {
                if (!bussinessTypeEntity.isCheckPc()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            } else if (orderEntity.getFromType() == ConstantStatus.FROM_CHANGE) {
                if (!bussinessTypeEntity.isCheckChange()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            } else if (orderEntity.getFromType() == ConstantStatus.FROM_UDISP) {
                if (!bussinessTypeEntity.isCheckSp()) {
                    unCheckFinish(orderEntity, warehouseEntityList);
                    return;
                }
            }

            genOrderDetail(warehouseEntityList);
            checkMutiDi(orderEntity.getId());   //校验一个DI是否绑定多个产品ID
            orderEntity = orderService.findById(orderEntity.getId());
            if (orderEntity.getContrastStatus() != ConstantStatus.ORDER_CHECK_FAIL) {
                orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_CHECK);
            }
        }
    }

    //无需校验，已完成
    public void unCheckFinish(OrderEntity orderEntity, List<WarehouseEntity> warehouseEntityList) {
        genOrderDetail(warehouseEntityList);
        checkMutiDi(orderEntity.getId());   //校验一个DI是否绑定多个产品ID
        orderEntity = orderService.findById(orderEntity.getId());
        if (orderEntity.getContrastStatus() != ConstantStatus.ORDER_CHECK_FAIL) {
            List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterAllByOrderIdFk(orderEntity.getId());
            for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                erpOrderEntity.setErpCount(erpOrderEntity.getCount());
                erpOrderEntity.setReCount(erpOrderEntity.getCount());
                myErpOrderService.updateErpOrder(erpOrderEntity);
            }
            orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_SUCCESS);
            invProductsTrService.genInvProducts(orderEntity.getId());
            printOrder(orderEntity.getId());
        }
    }

    //校验是否自带供应商
    public String checkOriginExitSup(WarehouseEntity warehouseEntity) {
        if (ConstantType.TYPE_PUT.equals(warehouseEntity.getMainAction())) {
            return warehouseEntity.getFromCorpId();
        } else if (ConstantType.TYPE_OUT.equals(warehouseEntity.getMainAction())) {
            if (warehouseEntity.getSupId() != null && !"".equals(warehouseEntity.getSupId())) {
                return warehouseEntity.getSupId();
            }
        }
        return null;
    }

    //校验库存是否存在供应商
    public String checkInvExitSup(WarehouseEntity warehouseEntity) {
        String code = warehouseEntity.getCode();
        UdiInfoEntity udiInfoEntity = invProductsTrService.getMinDi(warehouseEntity);
        if (udiInfoEntity.getDiType() == ConstantStatus.DITYPE_PACK || udiInfoEntity.getDiType() == ConstantStatus.DITYPE_SYDY
                || (udiInfoEntity.getSjcpbm() != null && !"".equals(udiInfoEntity.getSjcpbm()))) {
            warehouseEntity.setNameCode(udiInfoEntity.getNameCode());
            code = FilterUdiUtils.transGlxpNoSerStr(warehouseEntity);
        }
        FilterInvProductDetailRequest filterInvProductDetailRequest = new FilterInvProductDetailRequest();
        filterInvProductDetailRequest.setCode(code);
        filterInvProductDetailRequest.setMainAction(ConstantType.TYPE_PUT);
        List<InvProductDetailEntity> invProductDetailEntities = invProductDetailService.filterInvProduct(filterInvProductDetailRequest);
        if (invProductDetailEntities != null && invProductDetailEntities.size() > 0) {
            //现在默认库存每个条码只存在一个供应商
            InvProductDetailEntity invProductDetailEntity = invProductDetailEntities.get(0);
            String supId = invProductDetailEntity.getSupId();
            for (InvProductDetailEntity temp : invProductDetailEntities) {
                if (!supId.equals(temp.getSupId())) {
                    return Constant.MUTI;
                }
            }
            return supId;
        }
        return null;
    }

//    //校验是否存在供应商
//    public String checkExitSup(String relId) {
//        //校验条码关联供应商
//        FilterCompanyProductRelevanceRequest filterCompanyProductRelevanceRequest = new FilterCompanyProductRelevanceRequest();
//        filterCompanyProductRelevanceRequest.setUdiRlIdFk(relId);
//        List<CompanyProductRelevanceEntity> companyProductRelevanceEntities = companyProductRelevanceService.filterCompanyProductRelevance(filterCompanyProductRelevanceRequest);
//        if (companyProductRelevanceEntities != null && companyProductRelevanceEntities.size() == 1) {
//            CompanyProductRelevanceEntity companyProductRelevanceEntity = companyProductRelevanceEntities.get(0);
//            CompanyEntity companyEntity = companyService.findCompany(companyProductRelevanceEntity.getCustomerId());
//            return companyEntity.getUnitIdFk();
//        } else if (companyProductRelevanceEntities != null && companyProductRelevanceEntities.size() > 1) {
//            return "MUTI";      //绑定多个供应商
//        }
//        return null;
//    }

    public void genOrderDetail(List<WarehouseEntity> warehouseEntityList) {
        inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, warehouseEntityList);
    }


    public void checkMutiDi(String orderId) {
        //校验是否di绑定多个产品ID
        OrderEntity orderEntity = orderService.findById(orderId);
        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());
        FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
        filterErpOrderRequest.setOrderId(orderId);
        List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterAllMyErpOrder(filterErpOrderRequest);
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                WarehouseEntity warehouseEntity = warehouseService.findOneByNameCode(erpOrderEntity.getOrderIdFk(), erpOrderEntity.getNameCode());
                if (warehouseEntity != null && warehouseEntity.getStatus() == ConstantStatus.ORDER_DETAIL_DI_MUTI) {
                    erpOrderEntity.setBindStatus(ConstantStatus.ORDER_DETAIL_DI_MUTI);
                    myErpOrderService.updateErpOrder(erpOrderEntity);
                    if (warehouseEntity.getRelId() == null) {
                        orderEntity.setRemark("该产品DI绑定多个产品ID，请在扫码单据详情绑定对应产品ID");
                        orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                        orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                        orderService.updateOrder(orderEntity);
                    }

                } else {
                    if (!bussinessTypeEntity.isInnerOrder()) {
                        if (erpOrderEntity.getSupId() == null) {
                            erpOrderEntity.setSupId(null);
                            erpOrderEntity.setBindStatus(ConstantStatus.ORDER_DETAIL_SUP_NONE);
                            orderEntity.setRemark("该产品未绑定供应商");
                            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                            orderService.updateOrder(orderEntity);
                        } else if (Constant.MUTI.equals(erpOrderEntity.getSupId())) {
                            erpOrderEntity.setBindStatus(ConstantStatus.ORDER_DETAIL_SUP_MUTI);
                            orderEntity.setRemark("该产品绑定多个供应商");
                            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                        } else {
                            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_CHECK);
                            erpOrderEntity.setBindStatus(ConstantStatus.ORDER_DETAIL_SUP_ONE);
                        }
                    }
                    myErpOrderService.updateErpOrder(erpOrderEntity);
                }

            }

        }
    }


    //校验ERP单
    @Async
    public void checkOrder() {

        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setStatus(ConstantStatus.ORDER_STATUS_CHECK);
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        for (int i = 0; i < orderEntities.size(); i++) {
            OrderEntity orderEntity = orderEntities.get(i);
            checkOrder(orderEntity);
        }
    }

    public void checkOrder(OrderEntity orderEntity) {
        orderEntity = orderService.findById(orderEntity.getId());
        List<String> erpIds = DataTransUtil.strToErpList(orderEntity.getErpFk());
        List<WarehouseEntity> transData = warehouseService.findByReceiptId(orderEntity.getId());
        if (erpIds != null && erpIds.size() > 0) {
            myErpOrderService.deleteByOrderId(orderEntity.getId());
            inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, transData);
            BasicThirdSysDetailEntity basicThirdSysDetailEntity = bussinessTypeService.findByActionKey(orderEntity.getAction(), "orderQueryUrl");
            if (basicThirdSysDetailEntity.getThirdSysFk() == null && basicThirdSysDetailEntity.getLocalAction() != null) {
                List<ErpOrderEntity> erpOrderEntities = thrDataService.getStockOrders(erpIds);
                checkThirdOrder(erpOrderEntities, orderEntity, transData);
            } else if (basicThirdSysDetailEntity != null && basicThirdSysDetailEntity.getEnabled()) {
                if (basicThirdSysDetailEntity.getFromType() == 0) {
                    if (basicThirdSysDetailEntity.getValue() == null) {//实时拉取
                        orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                        orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                        orderEntity.setRemark("获取ERP出入库单据接口地址未定义");
                        orderService.updateOrder(orderEntity);
                    } else {
                        String erpOrderUrl = basicThirdSysDetailEntity.getValue();
                        List<ErpOrderEntity> cloudErps = new ErpOrderClient(erpOrderUrl).getErpOrder(erpIds, orderEntity.getAction());

                        if (basicThirdSysDetailEntity.getLocalAction() != null) {
                            List<ErpOrderEntity> erpOrderEntities = thrDataService.getStockOrders(erpIds);
                            if (cloudErps == null) {
                                cloudErps = new ArrayList<>();
                            }
                            cloudErps.addAll(erpOrderEntities);
                            checkThirdOrder(erpOrderEntities, orderEntity, transData);

                        } else {
                            checkThirdOrder(cloudErps, orderEntity, transData);
                        }

                    }
                } else {    //缓存表拉取
                    List<ErpOrderEntity> erpOrderEntities = thrOrderService.filterAllOrders(erpIds, basicThirdSysDetailEntity.getThirdSysFk());
                    if (basicThirdSysDetailEntity.getLocalAction() != null) {
                        List<ErpOrderEntity> temps = thrDataService.getStockOrders(erpIds);
                        if (erpOrderEntities == null) {
                            erpOrderEntities = new ArrayList<>();
                        }
                        erpOrderEntities.addAll(temps);
                        checkThirdOrder(erpOrderEntities, orderEntity, transData);

                    } else {
                        checkThirdOrder(erpOrderEntities, orderEntity, transData);
                    }
                }

            } else {
                orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                orderEntity.setRemark("第三方出入库单据未启用");
                orderService.updateOrder(orderEntity);
                myErpOrderService.deleteByOrderId(orderEntity.getId());
                inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, transData);
            }


        } else {
            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
            orderEntity.setRemark("未选入第三方出入库单据进行校验，请手动选择");
            orderService.updateOrder(orderEntity);
            myErpOrderService.deleteByOrderId(orderEntity.getId());
            inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, transData);
        }
    }

    public void checkThirdOrder(List<ErpOrderEntity> cloudErps, OrderEntity orderEntity, List<WarehouseEntity> transData) {
        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());
        if (cloudErps != null && cloudErps.size() > 0) {
            //获取需要校验的的第三方单据
            List<ErpOrderEntity> returnOrders = contrastErpUtil.transErp(cloudErps, udiRelevanceService, orderEntity.getId(), bussinessTypeEntity);
            ErpOrderEntity temp = returnOrders.get(0);
            for (int i = 0; i < returnOrders.size(); i++) {
                if (!temp.getCompanyname().equals(returnOrders.get(i).getCompanyname())) {
                    orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                    orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                    orderEntity.setRemark("选择Erp订单往来单位不一致");
                    orderService.updateOrder(orderEntity);
                    return;
                }
            }
//            myErpOrderService.deleteByOrderId(orderEntity.getId());
//            myErpOrderService.insertErpOrders(returnOrders);
            contrastErpUtil.contrastErp(udiInfoService, udiRelevanceService, myErpOrderService, orderService, transData, systemParamConfigService, returnOrders);
        } else {
            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
            orderEntity.setRemark("获取ERP出入库单据失败");
            orderService.updateOrder(orderEntity);
            myErpOrderService.deleteByOrderId(orderEntity.getId());
            inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, transData);
        }
    }

    //重码校验错误重新提交
    @Async
    public void repeatCommit(CommitRequest commitRequest) {
        OrderEntity orderEntity = orderService.findById(commitRequest.getOrderId());
        if (orderEntity != null) {
            transInout(orderEntity);
            WebSocketServer.sendInfo("重码校验已完成，请刷新查看！", null);
        }
    }

    //重新校验
    @Async
    public void repeatCheck(String orderId) {
        OrderEntity orderEntity = orderService.findById(orderId);
//        List<WarehouseEntity> warehouseEntityList = warehouseService.findByReceiptId(orderEntity.getId());
//        inoutUtils.generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, warehouseEntityList);
        checkOrder(orderEntity);
        WebSocketServer.sendInfo("单据:" + orderId + "校验已完成，请刷新查看！", null);
    }

    //校验ERP单后，订单详情转入等待打印
    @Async
    public void printOrder(String orderId) {
        syncprintOrder(orderId);
    }


    //单据校验成功后，更改或生成单据状态
    public void syncprintOrder(String orderId) {
        OrderEntity orderEntity = orderService.findById(orderId);
        if (orderEntity.getStatus() == ConstantStatus.ORDER_STATUS_SUCCESS && orderEntity.getErpFk() == null
        ) {

            //查询单据校验后生成的单据详情
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setOrderId(orderId);
            List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterAllMyErpOrder(filterErpOrderRequest);
            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());

            //生成业务单据详情
            List<StockOrderDetailEntity> stockOrderDetailEntities = new ArrayList<>();
            if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                boolean isError = false;
                for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                    StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
                    stockOrderDetailEntity.setProductId(erpOrderEntity.getGoodsid());
                    stockOrderDetailEntity.setProductName(erpOrderEntity.getGoodsname());

                    if (erpOrderEntity.getGoodsid() == null)
                        stockOrderDetailEntity.setProductId(erpOrderEntity.getNameCode());
                    if (erpOrderEntity.getGoodsname() == null)
                        stockOrderDetailEntity.setProductName(erpOrderEntity.getCoName());
                    stockOrderDetailEntity.setSpec(erpOrderEntity.getPackSpec());
                    stockOrderDetailEntity.setBatchNo(erpOrderEntity.getBatchNo());
                    stockOrderDetailEntity.setProductDate(erpOrderEntity.getProductDate());
                    stockOrderDetailEntity.setExpireDate(erpOrderEntity.getExpireDate());
//                    stockOrderDetailEntity.setYlqxzcrbarmc(erpOrderEntity.get());
                    if (erpOrderEntity.getCount() != null) {
                        stockOrderDetailEntity.setSweepCount(erpOrderEntity.getCount() + "");
                        stockOrderDetailEntity.setReCount(erpOrderEntity.getCount() + "");
                        stockOrderDetailEntity.setCount(erpOrderEntity.getCount() + "");
                    }
                    if (erpOrderEntity.getReCount() != null) {
                        stockOrderDetailEntity.setReCount(erpOrderEntity.getReCount() + "");
                    } else {
                        stockOrderDetailEntity.setReCount(erpOrderEntity.getCount() + "");
                    }
                    if (erpOrderEntity.getErpCount() != null) {
                        stockOrderDetailEntity.setCount(erpOrderEntity.getErpCount() + "");
                    }
//                    FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
//                    filterUdiInfoRequest.setNameCode(erpOrderEntity.getNameCode());
//                    List<UdiRelevanceResponse> udiRelevanceEntities = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
//                    if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 1 && erpOrderEntity.getBindRlFk() == null) {
//                        isError = true;
//                        erpOrderEntity.setBindStatus(ConstantStatus.ORDER_DETAIL_DI_MUTI);
//                        myErpOrderService.updateErpOrder(erpOrderEntity);
//                    } else if (udiRelevanceEntities.size() == 1) {
//                        UdiRelevanceResponse udiRelevanceEntity = udiRelevanceEntities.get(0);
//                        setThridName(bussinessTypeEntity, stockOrderDetailEntity, udiRelevanceEntity);
//
//                    } else if (erpOrderEntity.getBindRlFk() != null) {
//                        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectById(erpOrderEntity.getBindRlFk());
//                        UdiRelevanceResponse udiRelevanceResponse = new UdiRelevanceResponse();
//                        BeanUtils.copyProperties(udiRelevanceEntity, udiRelevanceResponse);
//                        udiRelevanceResponse.setId(Integer.parseInt(udiRelevanceEntity.getId()));
//                        setThridName(bussinessTypeEntity, stockOrderDetailEntity, udiRelevanceResponse);
//                        erpOrderEntity.setGoodsid(stockOrderDetailEntity.getProductId());
//                        erpOrderEntity.setGoodsname(stockOrderDetailEntity.getProductName());
//                        myErpOrderService.updateErpOrder(erpOrderEntity);
//                    }

                    stockOrderDetailEntities.add(stockOrderDetailEntity);
                }
                if (!isError) {

                    StockOrderEntity stockOrderEntity = genStockOrder(orderEntity, bussinessTypeEntity);
                    for (StockOrderDetailEntity stockOrderDetailEntity : stockOrderDetailEntities) {
                        stockOrderDetailEntity.setOrderIdFk(stockOrderEntity.getId());
                    }
                    stockOrderDetailService.insertStockOrderDetails(stockOrderDetailEntities);
                    orderEntity.setRemark("");
                    orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                    orderService.updateOrder(orderEntity);
                } else {
                    orderEntity.setRemark("该产品DI绑定多个产品ID，请在扫码单据详情绑定对应产品ID");
                    orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                    orderService.updateOrder(orderEntity);
                }
            }
        } else if (orderEntity.getContrastStatus() == ConstantStatus.ORDER_CHECK_SUCCESS) {
            String[] erpIds = orderEntity.getErpFk().split(",");
            if (erpIds != null && erpIds.length > 0) {
                for (int i = 0; i < erpIds.length; i++) {
                    StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
                    stockOrderFilterRequest.setBillNo(erpIds[i]);
                    StockOrderEntity stockOrderEntity = stockOrderService.findOne(stockOrderFilterRequest);
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                    stockOrderService.updateById(stockOrderEntity);
                }
            }

        }
    }


    public StockOrderEntity genStockOrder(OrderEntity orderEntity, BussinessTypeEntity bussinessTypeEntity) {
        //生成业务单据记录
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        stockOrderEntity.setBillNo(orderEntity.getId());
        stockOrderEntity.setBillType(bussinessTypeEntity.getLocalAction());//todo
        stockOrderEntity.setBilldate(orderEntity.getActDate());
        stockOrderEntity.setUnitIdFk(orderEntity.getFromCorpId());
        stockOrderEntity.setSourceType(ConstantStatus.ST_SCAN_ADD);
        stockOrderEntity.setCorpId(orderEntity.getFromCorpId());
        if (bussinessTypeEntity.isSecCheckEnable()) {
            stockOrderEntity.setStatus(ConstantStatus.SORDER_DELIVER);
            if (orderEntity.getFromType().intValue() == ConstantStatus.FROM_UDIMS) {
                if (!bussinessTypeEntity.isSecCheckUdims()) {
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                }
            } else if (orderEntity.getFromType().intValue() == ConstantStatus.FROM_WEBNEW) {
                if (!bussinessTypeEntity.isSecCheckWebNew()) {
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                }
            } else if (orderEntity.getFromType().intValue() == ConstantStatus.FROM_PDAED) {
                if (!bussinessTypeEntity.isSecCheckPdaEd()) {
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                }
            } else if (orderEntity.getFromType().intValue() == ConstantStatus.FROM_PDAUN) {
                if (!bussinessTypeEntity.isSecCheckPdaUn()) {
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                }
            } else if (orderEntity.getFromType().intValue() == ConstantStatus.FROM_PC) {
                if (!bussinessTypeEntity.isSecCheckPc()) {
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                }
            } else if (orderEntity.getFromType().intValue() == ConstantStatus.FROM_CHANGE) {
                if (!bussinessTypeEntity.isSecCheckChange()) {
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                }
            }
        } else {
            stockOrderEntity.setStatus(ConstantStatus.SORDER_DELIVER);
        }

        stockOrderEntity.setCorpName(orderEntity.getFromCorp());
        stockOrderEntity.setOrderIdFk(orderEntity.getId());
        stockOrderService.insertStockOrder(stockOrderEntity);

        //删除关联的业务单据详情
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        stockOrderFilterRequest.setBillNo(stockOrderEntity.getBillNo());
        stockOrderEntity = stockOrderService.findOne(stockOrderFilterRequest);
        stockOrderDetailService.deleteByOrderIdFk(stockOrderEntity.getId());

        return stockOrderEntity;
    }

}
